package dk.nsp.idsas.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.util.AuditLogger;
import org.apache.commons.codec.binary.Base64;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;

@Configuration
@PropertySources({
        @PropertySource("classpath:/META-INF/maven/dk.nsp.idsas/idsas-lookup-web/pom.properties"),
        @PropertySource(value = "idsas-lookup.properties", encoding = "ISO-8859-1"),
        @PropertySource(value = "idsas-patient-id-salt.properties", encoding = "ISO-8859-1")
        // Bemærkning: I Java 8 er properties altid i ISO-8859-1, men fremadrettet skulle de gerne kunne være UTF-8.
})
public class WebSetup {

    @Value("${idsas.allowed.client.system.names}")
    private String[] allowedClientSystemNames;

    @Value("${idsas.patient.id.saltPrefix:#{null}}")
    private String patientIdSaltPrefix;

    @Value("${idsas.patient.id.saltPostfix:#{null}}")
    private String patientIdSaltPostfix;

    @Bean
    public LookupService lookupService(BlurringDao blurringDao, AuditLogger auditLogger) {
        // If patientIdSaltEncoded is null or empty
        if (patientIdSaltPrefix == null || patientIdSaltPrefix.isEmpty() || patientIdSaltPostfix == null || patientIdSaltPostfix.isEmpty()) {
            throw new IllegalStateException("patientIdSalt cannot be empty or null");
        }

        return new LookupService(blurringDao, patientIdSaltPrefix, patientIdSaltPostfix, auditLogger, allowedClientSystemNames);
    }
}
